﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace BBSensorDriverDemo
{
    public partial class Form1 : Form
    {
        // BB Sensor Interface
        BBSensorDriver.IBBSensorDriver bbSensor;
        List<string> deviceStringList = new List<string>();
        public Form1()
        {
            InitializeComponent();
        }

        private void buttonFindDevices_Click(object sender, EventArgs e)
        {
            // Create access to our sensor driver.
            bbSensor = new BBSensorDriver.BBSensorUsbDriver();

            // Clear current device list, if any.
            this.DeviceListBox.Items.Clear();
            this.textBoxLog.Clear();

            // Find devices.
            deviceStringList = new List<string>();
            Boolean status = bbSensor.Find(ref deviceStringList);

            if (status)
            {
                // Add to device list UI.
                foreach (string deviceString in deviceStringList)
                {
                    //Retrive device PID, VID and Serial number.
                    string deviceName = deviceString.Substring(4, deviceString.IndexOf("{") - 4);

                    this.DeviceListBox.Items.Add(deviceName);
                }

                //Enable Connect button
                this.buttonConnect.Enabled = true;
            }
            else
            {
                this.textBoxLog.AppendText("No sensor found.\n");

                //Disable Connect button
                this.buttonConnect.Enabled = false;
            }

            // Set selection to the top item.
            if (this.DeviceListBox.Items.Count > 0)
            {
                this.DeviceListBox.SelectedIndex = 0;
            }

        }
        private void buttonConnect_Click(object sender, EventArgs e)
        {
            string deviceName = "";
            string deviceString = "";

            this.textBoxLog.Clear();

            // Try to open the selected device.
            int selectedDeviceIndex = this.DeviceListBox.SelectedIndex;

            if (selectedDeviceIndex >= 0)
            {
                deviceName = this.DeviceListBox.Items[this.DeviceListBox.SelectedIndex].ToString();

                //Find device complete USB string
                foreach (string device in deviceStringList)
                {
                    if (device.IndexOf(deviceName) > -1)
                    {
                        deviceString = device;
                        break;
                    }
                }
            }
            else
            {
                this.buttonConnect.Enabled = false;
            }

            //Connect to sensor
            bool status = bbSensor.Open(deviceString, 22222); // timeout in ms

            if (status)
            {
                //Disable buttons
                this.buttonConnect.Enabled = false;
                this.buttonFindDevices.Enabled = false;

                //Enable buttons
                this.buttonDisconnect.Enabled = true;
                this.buttonGetIDN.Enabled = true;
                this.buttonGetPower.Enabled = true;
                this.buttonGetFrequency.Enabled = true;
                this.buttonSetFrequency.Enabled = true;
                this.buttonSetListModeParams.Enabled = true;
                this.buttonListStart.Enabled = true;
            }
            else
            {
                this.buttonConnect.Enabled = false;
                this.textBoxLog.AppendText("Connect sensor error.\n");
            }
        }

        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            //Close connection to sensor.
            bbSensor.Close();

            //Enable buttons
            this.buttonConnect.Enabled = true;
            this.buttonFindDevices.Enabled = true;

            //Disable buttons
            this.buttonDisconnect.Enabled = false;
            this.buttonGetIDN.Enabled = false;
            this.buttonGetPower.Enabled = false;
            this.buttonGetFrequency.Enabled = false;
            this.buttonSetFrequency.Enabled = false;
            this.buttonSetListModeParams.Enabled = false;
            this.buttonListStart.Enabled = false;

            this.textBoxLog.AppendText("Sensor disconnected.\n");
        }

        private void buttonGetIDN_Click(object sender, EventArgs e)
        {
            //Send IDN? command and display the return string
            this.WriteAndRead("IDN?");
        }

        private void buttonGetPower_Click(object sender, EventArgs e)
        {
            //Send PWR? command and display the return power
            this.WriteAndRead("PWR?");
        }

        private void buttonSetFrequency_Click(object sender, EventArgs e)
        {
            //Set frequency to 1.5GHz
            //Send FREQ 1.5 command and display the return status
            this.WriteAndRead("FREQ 1.5");
        }

        private void buttonGetFrequency_Click(object sender, EventArgs e)
        {
            //Read current frequency setting
            //Send FREQ? command and display the return frequency
            this.WriteAndRead("FREQ?");
        }

        private string WriteAndRead(string command)
        {
            try
            {
                bbSensor.WriteLine(command);
                this.textBoxLog.AppendText("Sent: " + command + "\n");
                string readStr = bbSensor.ReadLine();
                this.textBoxLog.AppendText("Received: " + readStr + "\n");

                return readStr;
            }
            catch (Exception)
            {
                //Catch time out exception
                this.textBoxLog.AppendText("Command time out.\n");

                //Disconnect the sensor.
                this.buttonDisconnect_Click(this, null);

                return "";
            }
        }

        private bool isInListMode = false;
        private string currentMode = "";
        private void switchToListMode()
        {
            //Send CHMOD? command to store current mode
            currentMode = this.WriteAndRead("CHMOD?");

            //Send CHMOD 4 command switch to List Mode.
            this.WriteAndRead("CHMOD 4");
            isInListMode = true;
        }
        private void exitListMode()
        {
            //Restore current mode
            this.WriteAndRead("CHMOD " + currentMode);

            isInListMode = false;
        }
        private void buttonSetListModeParams_Click(object sender, EventArgs e)
        {
            //Switch to list mode if needed
            if(!isInListMode)
                switchToListMode();

            OpenFileDialog fileChooser = new OpenFileDialog();
            fileChooser.Multiselect = false;
            DialogResult result = fileChooser.ShowDialog();
            if (result == DialogResult.OK)
            {
                this.textBoxLog.AppendText("Selected file: " + fileChooser.FileName + "\n");

                //Send LOADLIST <dataSize> command
                FileInfo fileInfo = new FileInfo(fileChooser.FileName);
                string response = this.WriteAndRead("LOADLIST " + fileInfo.Length);
                if (response.CompareTo("OK") != 0)
                {
                    return;
                }

                //Send READY? command, and check confirmation to start upload
                response = this.WriteAndRead("READY?");
                if (response.CompareTo("READY") == 0)
                {
                    using (StreamReader r = new StreamReader(fileChooser.FileName))
                    {
                        int bufferSize = 512;
                        char[] buffer = new char[bufferSize];
                        this.textBoxLog.AppendText("Sending file data. Progress:");

                        //Read 512 byte data and send to sensor
                        while (r.ReadBlock(buffer, 0, bufferSize) > 0)
                        {
                            //send data to sensor
                            bbSensor.Write(new string(buffer));
                            response = bbSensor.ReadLine();
                            if (response.CompareTo("OK") != 0)
                            {
                                this.textBoxLog.AppendText("An error occurred while sending data. Response =  " + response + "\n");
                                return;
                            }
                        }
                        this.textBoxLog.AppendText("\n");
                    }

                    //Send READY? command, and check confirmation to complete the upload
                    response = this.WriteAndRead("READY?");
                    if (response.CompareTo("READY") == 0)
                    {
                        this.textBoxLog.AppendText("File data transfer complete!\n");
                    }
                    else
                    {
                        this.textBoxLog.AppendText("An error occurred while attemting to sync with the sensor after sending data.\n");
                    }
                }
                else
                {
                    this.textBoxLog.AppendText("An error occurred while attempting to sync with the sensor before sending data.\n");
                }
            }

        }

        private void buttonListStart_Click(object sender, EventArgs e)
        {
            //Switch to list mode if needed
            if (!isInListMode)
                switchToListMode();

            //Send LISTSTART command to start List Mode measurement
            string response = this.WriteAndRead("LISTSTART");
            if (response.Equals("OK"))
            {
                // Sensor has max 1 sec aperture time so it should take at least 10
                // measurements before a single read timeout occurs.
                int numTries = 4800; //48000 max measurements / 10 measurements per timeout = 4800

                // Sensor may take a long time to respond while it's
                // taking measurements, so we need to increase the amount
                // of time that we wait for a response
                do
                {
                    try
                    {
                        response = bbSensor.ReadLine();
                    }
                    catch (System.TimeoutException ex)
                    {
                        response = "Timeout occurred.";
                    }
                } while (response.Equals("Timeout occurred.") && (--numTries > 0));

                if (response.Equals("Timeout occurred."))
                {
                    MessageBox.Show("A timeout occurred while waiting for the sensor to send measurement data.");
                }
                else
                {
                    this.textBoxLog.AppendText(response + "\n");
                }
            }

            exitListMode();
        }

    }
}
